/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.energy.configs;

import de.tud.kom.p2psim.api.common.SimHost;
import de.tud.kom.p2psim.api.energy.EnergyConfiguration;
import de.tud.kom.p2psim.impl.energy.components.LoRaCommunicationEnergyComponent;

/**
 * Energy Configuration for the LoRa Component of a Heltec ESP32 LoRa device
 * @author Lukas Wehrstein
 * @version 1.0, 08.03.2022
 */
public class HeltecESP32LoRaConfig implements EnergyConfiguration<LoRaCommunicationEnergyComponent>{

	// Spreading Factor
	private int sf = 7;
	
	// transmission power in dBm
	private double transmissionPowerDBm = 7; // TODO LW set good default value
	
	private LoRaCommunicationEnergyComponent energyComponent;
	
	@Override
	public LoRaCommunicationEnergyComponent getConfiguredEnergyComponent(
			SimHost host) {
		
		// TODO
		energyComponent = new LoRaCommunicationEnergyComponent(calculateStateEnergyConsumptions());
		return energyComponent;
	}
	
	/**
	 * Calculate the energy consumption of the states
	 * @return a array of 4 doubles with the calculated energy consumption (in uW) for IDLE, SEND, RECEIVE, OFF
	 */
	public double[] calculateStateEnergyConsumptions() {
		// IDLE, SEND, RECEIVE, OFF
		// 375000uW = 5V*75mA (mal 1.6, da Zeit nur auf 0,43s statt 0,75s berechnet wird) => 600000
		double[] values = {0, 600000, 60000, 0};
		// TODO: calculate values depending on sf and transmissionPower
		return values;
	}

	@Override
	public String getHelp() {
		// TODO Auto-generated method stub
		return "HELP";
	}

	@Override
	public boolean isWellConfigured() {
		// TODO Auto-generated method stub
		return true;
	}
	
	/**
	 * Sets the Spreading Factor of the LoRa Component
	 * @param sf
	 */
	public void setSF(int sf) {
		this.sf = sf;
	}
	
	/**
	 * Sets the LoRa Transmission Power in dBm
	 * @param transmissionPowerDBm
	 */
	public void setTransmissionPowerDBm(double transmissionPowerDBm) {
		this.transmissionPowerDBm = transmissionPowerDBm;
	}

}
