/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.wsn;

import de.tud.kom.p2psim.api.energy.ComponentType;
import de.tud.kom.p2psim.api.energy.EnergyComponent;
import de.tud.kom.p2psim.impl.energy.components.OneStateEnergyComponent;
import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.Monitor;
import de.tudarmstadt.maki.simonstrator.api.Monitor.Level;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponentFactory;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.WSNSensorComponent;
import de.tudarmstadt.maki.simonstrator.api.Randoms;

/**
 * This class represents a generic sensor component
 * @author Lukas Wehrstein
 * @version 1.0, 31.01.2022
 */
public class GenericWSNSensor extends GenericWSNComponent implements WSNSensorComponent {

	private SensorType sensorType;
	
	private double data;
	
	/**
	 * Creates a generic Sensor component
	 * @param host
	 * @param sensorType
	 * @param energyComponent for energy simulation
	 */
	public GenericWSNSensor(Host host, SensorType sensorType, EnergyComponent energyComponent) {
		super(host, ComponentType.SENSOR, energyComponent);
		this.sensorType = sensorType;
		this.name += "." + sensorType.toString();
		this.data = 0;
	}
	
	public SensorType getSensorType() {
		return this.sensorType;
	}
	
	@Override
	public boolean testSensor() {
		return true;
	}
	
	@Override
	public double getSensorData() {
//		setActive(true);
//		getEnergyComponent().turnOn();
//		generateRandomData();
//		
//		// TODO LW: Needs to be adjusted with propper timing solution...
//		//wait 1 second until data is there
//		Event.scheduleWithDelay(Time.SECOND, new EventHandler() {
//
//			@Override
//			public void eventOccurred(Object content, int type) {
//				getEnergyComponent().turnOff();
//				setActive(false);
//			}
//			
//		}, this, 0);
//		
//		return data;
		if(!isActive()) {
			setActive(true);
		}
		generateRandomData();
		return this.data;
	}
	
	/**
	 * Generates random sensor data
	 */
	public void generateRandomData() {
		this.data = Randoms.getRandom(getClass()).nextDouble();
		//data = Math.random()*2 - 1;
		Monitor.log(getClass(), Level.DEBUG, "Generated Random Double: ", this.data);
	}
	
	@Override
	public int getSensorDataSize() {
		return Double.BYTES;
	}

	public static class Factory implements HostComponentFactory{

		SensorType sensorType;
		EnergyComponent energyComponent;
		double energyConsumptionUwatt;
		
		
		/**
		 * Type of the sensor
		 * @param sensorType
		 */
//		public void setSensorType(SensorType sensorType) {
//			System.out.println("set ST: " + sensorType);
//			this.sensorType = sensorType;
//		}
		
		public void setSensorType(String sensorType) {
			try {
				this.sensorType = SensorType.valueOf(sensorType.toUpperCase());
			}catch (Exception e) {
				Monitor.log(getClass(), Level.ERROR, "%s is not a valid SensorType. Using GENERIC instead", sensorType);
				this.sensorType = SensorType.GENERIC;
			}
			
			// System.out.println("set ST_string: " + sensorType);
			
		}
		
		
		/**
		 * Defines the energy consumption of this sensor
		 * @param energyConsumptionUwatt
		 */
		public void setEnergyConsumptionUwatt(double energyConsumptionUwatt) {
			Monitor.log(getClass(), Level.DEBUG, "EnergyConsumption of this %s sensor was set to %s uW", sensorType, this.energyConsumptionUwatt);
			this.energyConsumptionUwatt = energyConsumptionUwatt;
		}
		
		@Override
		public HostComponent createComponent(Host host) {
			this.energyComponent = new OneStateEnergyComponent(energyConsumptionUwatt, ComponentType.SENSOR, false);
			return new GenericWSNSensor(host, sensorType, energyComponent);
		}
		
	}

}
